package org.xbib.interlibrary.catalog.oracle.aleph;

import java.text.MessageFormat;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.xbib.interlibrary.catalog.builder.util.FormatUtil;
import org.xbib.metrics.api.Count;
import org.xbib.metrics.common.CountMetric;
import org.xbib.metrics.common.Meter;

public class AlephMetric {

    private static final Logger logger = Logger.getLogger(AlephMetric.class.getName());

    private ScheduledExecutorService service;

    private ScheduledFuture<?> future;

    private MetricRunnable metricRunnable;

    private Meter totalMeter;

    private Count bytesCount;

    public AlephMetric() {
    }

    public void start() {
        this.service = Executors.newScheduledThreadPool(2);
        this.metricRunnable = new MetricRunnable();
        this.future = service.scheduleAtFixedRate(metricRunnable, 0L, 10, TimeUnit.SECONDS);
        this.totalMeter =  new Meter(service);
        this.bytesCount = new CountMetric();
        totalMeter.start(5L);
    }

    public void inc() {
        totalMeter.mark();
    }

    public void incBytes(long l) {
        bytesCount.inc(l);
    }

    public void stop() {
        totalMeter.stop();
    }

    public void close() {
        metricRunnable.run();
        totalMeter.stop();
        totalMeter.shutdown();
        this.future.cancel(true);
        service.shutdownNow();
        logger.log(Level.INFO, "metric closed");
    }

    private class MetricRunnable implements Runnable {

         private final Logger logger = Logger.getLogger(MetricRunnable.class.getName());

            @Override
            public void run() {
                logMeter("docs", totalMeter, bytesCount);
            }

            private void logMeter(String name, Meter meter, Count bytesCount) {
                if (meter == null) {
                    return;
                }
                if (bytesCount == null) {
                    return;
                }
                long count = meter.getCount();
                long bytes = bytesCount.getCount();
                long elapsed = meter.elapsed() / 1000000; // nano to millis
                double dps = count * 1000.0 / elapsed;
                double avg = bytes / (count + 1.0); // avoid div by zero
                double bps = bytes * 1000.0 / elapsed;
                logger.log(Level.INFO, MessageFormat.format("{0}: {1}, {2} ms = {3}, {4} = {5}, {6} = {7} avg, {8} = {9}, {10} = {11}",
                        name,
                        count,
                        elapsed,
                        FormatUtil.formatDurationWords(elapsed, true, true),
                        bytes,
                        FormatUtil.formatSize(bytes),
                        avg,
                        FormatUtil.formatSize(avg),
                        dps,
                        FormatUtil.formatDocumentSpeed(dps),
                        bps,
                        FormatUtil.formatSpeed(bps)
                ));
            }
     }
}
